"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Invoice = void 0;
const jsonwebtoken_1 = __importDefault(require("jsonwebtoken"));
const qrcode_1 = __importDefault(require("qrcode"));
const InvoicePDF_1 = require("./InvoicePDF");
const i18next_1 = __importDefault(require("i18next"));
const i18next_fs_backend_1 = __importDefault(require("i18next-fs-backend"));
const node_path_1 = __importDefault(require("node:path"));
class Invoice {
    invoice;
    privateKey;
    subtotalWithoutTax = 0;
    amountDue = 0;
    /**
     * @param {InvoiceInterface} invoice Invoice information
     * @param {Secret} privateKey prime256v1 private key
     */
    constructor(invoice, privateKey) {
        this.invoice = invoice;
        this.privateKey = privateKey;
        for (const line of this.invoice.lines) {
            this.subtotalWithoutTax += line.quantity * line.unitPrice;
            this.amountDue += line.quantity * line.unitPrice * (1 + line.tax);
        }
    }
    /**
     * Sign payload containing invoice information with private key
     * @param {InvoiceSignedPayload} signedPayload The payload to be signed
     * @param {Secret} privateKey The secret key to use
     * @return {string} The signed JWT token
     */
    static signJwt(signedPayload, privateKey) {
        return jsonwebtoken_1.default.sign(signedPayload, privateKey, { algorithm: 'ES256' });
    }
    /**
     * Prove the authenticity of a token by verifying its signature
     * @param {string} token The token to be verified
     * @param {Secret} key The issuer's public key
     * @return {InvoiceSignedPayload} The verified payload
     */
    static verifyJwt(token, key) {
        return jsonwebtoken_1.default.verify(token, key, { algorithms: ['ES256'] });
    }
    /**
     * Generate a signed token containing the instance information
     * @return {string} The signed JWT token
     */
    createJwt() {
        const { seller, client, lines, currency, reference, date, dueDate, payment } = this.invoice;
        return Invoice.signJwt({
            iss: `${seller.name} (${seller.identifier})`,
            sub: client.name,
            iat: Math.floor(date.getTime() / 1e3),
            dueDate: Math.floor(dueDate.getTime() / 1e3),
            amt: this.amountDue,
            curr: currency,
            qty: lines.reduce((q, l) => q + l.quantity, 0),
            line: lines.length,
            ref: reference,
            pay: payment
        }, this.privateKey);
    }
    /**
     * Generate a signed token and convert it to a QR Code PNG image
     * @return {Promise<Buffer>}
     */
    async createQRCodeBuffer() {
        return qrcode_1.default.toBuffer(this.createJwt(), { errorCorrectionLevel: 'L' });
    }
    /**
     * Generate the final PDF file containing the cryptographic stamp of authenticity
     * @return {Promise<PDFKit.PDFDocument>}
     */
    async generatePDF() {
        await i18next_1.default.use(i18next_fs_backend_1.default).init({
            lng: this.invoice.language,
            fallbackLng: 'en',
            backend: {
                loadPath: node_path_1.default.join(__dirname, '../langs/{{lng}}.json'),
                jsonIndent: 2
            }
        });
        return new InvoicePDF_1.InvoicePDF(this, i18next_1.default).generate();
    }
}
exports.Invoice = Invoice;
